@@ -87,7 +87,9 @@ static char *head_strings[] = {
     "BEGIN PGP PUBLIC KEY BLOCK",
     "BEGIN PGP SIGNATURE",
     "BEGIN PGP SIGNED MESSAGE",
-    "BEGIN PGP ARMORED FILE",
+    "BEGIN PGP ARMORED FILE",       /* gnupg extension */
+    "BEGIN PGP PRIVATE KEY BLOCK",
+    "BEGIN PGP SECRET KEY BLOCK",   /* only used by pgp2 */
     NULL
 };
 static char *tail_strings[] = {
@@ -96,6 +98,8 @@ static char *tail_strings[] = {
     "END PGP SIGNATURE",
     "END dummy",
     "END PGP ARMORED FILE",
+    "END PGP PRIVATE KEY BLOCK",
+    "END PGP SECRET KEY BLOCK",
     NULL
 };
 
@@ -153,8 +157,8 @@ is_armored( byte *buf )
     pkttype =  ctb & 0x40 ? (ctb & 0x3f) : ((ctb>>2)&0xf);
     switch( pkttype ) {
       case PKT_MARKER:
-      case PKT_PUBLIC_CERT:
-      case PKT_SECRET_CERT:
+      case PKT_PUBLIC_KEY:
+      case PKT_SECRET_KEY:
       case PKT_PUBKEY_ENC:
       case PKT_SIGNATURE:
       case PKT_COMMENT:
@@ -321,7 +325,7 @@ find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
 	    if( n < buflen || c == '\n' ) {
 		if( n && buf[0] != '\r') { /* maybe a header */
 		    if( strchr( buf, ':') ) { /* yes */
-			int hashes;
+			int hashes=0;
 			if( buf[n-1] == '\r' )
 			    buf[--n] = 0;
 			if( opt.verbose ) {
@@ -818,13 +822,19 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		else if( (c=iobuf_get(a)) == -1 )
 		    break;
 	    } while( ++idx < 4 );
-	    if( c == -1 )
+	    if( c == -1 ) {
 		log_error("premature eof (in CRC)\n");
-	    else if( idx != 4 )
+		rc = G10ERR_INVALID_ARMOR;
+	    }
+	    else if( idx != 4 ) {
 		log_error("malformed CRC\n");
-	    else if( mycrc != afx->crc )
+		rc = G10ERR_INVALID_ARMOR;
+	    }
+	    else if( mycrc != afx->crc ) {
 		log_error("CRC error; %06lx - %06lx\n",
 				    (ulong)afx->crc, (ulong)mycrc);
+		rc = G10ERR_INVALID_ARMOR;
+	    }
 	    else {
 		rc = 0;
 	      #if 0
@@ -839,10 +849,14 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		}
 		if( rc == -1 )
 		    rc = 0;
-		else if( rc == 2 )
+		else if( rc == 2 ) {
 		    log_error("premature eof (in Trailer)\n");
-		else
+		    rc = G10ERR_INVALID_ARMOR;
+		}
+		else {
 		    log_error("error in trailer line\n");
+		    rc = G10ERR_INVALID_ARMOR;
+		}
 	      #endif
 	    }
 	}
@@ -984,7 +998,8 @@ armor_filter( void *opaque, int control,
 	    iobuf_writestr(a, "-----\n");
 	    iobuf_writestr(a, "Version: GNUPG v"  VERSION " ("
 					    PRINTABLE_OS_NAME ")\n");
-	    iobuf_writestr(a, "Comment: This is an alpha version!\n");
+	    iobuf_writestr(a,
+		"Comment: Get GNUPG from ftp://ftp.guug.de/pub/gcrypt/\n");
 	    if( afx->hdrlines )
 		iobuf_writestr(a, afx->hdrlines);
 	    iobuf_put(a, '\n');
