@@ -43,6 +43,7 @@ static int  copy_packet( IOBUF inp, IOBUF out, int pkttype,
 					       unsigned long pktlen );
 static void skip_packet( IOBUF inp, int pkttype, unsigned long pktlen );
 static void skip_rest( IOBUF inp, unsigned long pktlen );
+static void *read_rest( IOBUF inp, ulong *r_pktlen );
 static int  parse_symkeyenc( IOBUF inp, int pkttype, unsigned long pktlen,
 							     PACKET *packet );
 static int  parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen,
@@ -51,7 +52,7 @@ static int  parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
 							 PKT_signature *sig );
 static int  parse_onepass_sig( IOBUF inp, int pkttype, unsigned long pktlen,
 							PKT_onepass_sig *ops );
-static int  parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
+static int  parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 				      byte *hdr, int hdrlen, PACKET *packet );
 static int  parse_user_id( IOBUF inp, int pkttype, unsigned long pktlen,
 							   PACKET *packet );
@@ -59,11 +60,11 @@ static int  parse_comment( IOBUF inp, int pkttype, unsigned long pktlen,
 							   PACKET *packet );
 static void parse_trust( IOBUF inp, int pkttype, unsigned long pktlen );
 static int  parse_plaintext( IOBUF inp, int pkttype, unsigned long pktlen,
-								PACKET *pkt );
+					       PACKET *packet, int new_ctb);
 static int  parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen,
-							   PACKET *packet );
+					       PACKET *packet, int new_ctb );
 static int  parse_encrypted( IOBUF inp, int pkttype, unsigned long pktlen,
-							   PACKET *packet );
+					       PACKET *packet, int new_ctb);
 
 
 static unsigned short
@@ -96,6 +97,19 @@ set_packet_list_mode( int mode )
     return old;
 }
 
+static void
+unknown_pubkey_warning( int algo )
+{
+    static byte unknown_pubkey_algos[256];
+
+    algo &= 0xff;
+    if( !unknown_pubkey_algos[algo] ) {
+	if( opt.verbose )
+	    log_info("can't handle public key algorithm %d\n", algo );
+	unknown_pubkey_algos[algo] = 1;
+    }
+}
+
 /****************
  * Parse a Packet and return it in packet
  * Returns: 0 := valid packet in pkt
@@ -176,6 +190,8 @@ skip_some_packets( IOBUF inp, unsigned n )
     return rc;
 }
 
+
+
 /****************
  * Parse packet. Set the variable skip points to to 1 if the packet
  * should be skipped; this is the case if either there is a
@@ -187,31 +203,35 @@ static int
 parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
        int *skip, IOBUF out, int do_skip )
 {
-    int rc, c, ctb, pkttype, lenbytes;
+    int rc=0, c, ctb, pkttype, lenbytes;
     unsigned long pktlen;
     byte hdr[8];
     int hdrlen;
-    int pgp3 = 0;
+    int new_ctb = 0;
 
     *skip = 0;
     assert( !pkt->pkt.generic );
     if( retpos )
 	*retpos = iobuf_tell(inp);
-    if( (ctb = iobuf_get(inp)) == -1 )
-	return -1;
+    if( (ctb = iobuf_get(inp)) == -1 ) {
+	rc = -1;
+	goto leave;
+    }
     hdrlen=0;
     hdr[hdrlen++] = ctb;
     if( !(ctb & 0x80) ) {
 	log_error("%s: invalid packet (ctb=%02x)\n", iobuf_where(inp), ctb );
-	return G10ERR_INVALID_PACKET;
+	rc = G10ERR_INVALID_PACKET;
+	goto leave;
     }
     pktlen = 0;
-    pgp3 = !!(ctb & 0x40);
-    if( pgp3 ) {
+    new_ctb = !!(ctb & 0x40);
+    if( new_ctb ) {
 	pkttype =  ctb & 0x3f;
 	if( (c = iobuf_get(inp)) == -1 ) {
 	    log_error("%s: 1st length byte missing\n", iobuf_where(inp) );
-	    return G10ERR_INVALID_PACKET;
+	    rc = G10ERR_INVALID_PACKET;
+	    goto leave;
 	}
 	hdr[hdrlen++] = c;
 	if( c < 192 )
@@ -220,24 +240,25 @@ parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
 	    pktlen = (c - 192) * 256;
 	    if( (c = iobuf_get(inp)) == -1 ) {
 		log_error("%s: 2nd length byte missing\n", iobuf_where(inp) );
-		return G10ERR_INVALID_PACKET;
+		rc = G10ERR_INVALID_PACKET;
+		goto leave;
 	    }
 	    hdr[hdrlen++] = c;
 	    pktlen += c + 192;
 	}
-	else if( c < 255 ) {
+	else if( c == 255 ) {
 	    pktlen  = (hdr[hdrlen++] = iobuf_get_noeof(inp)) << 24;
 	    pktlen |= (hdr[hdrlen++] = iobuf_get_noeof(inp)) << 16;
 	    pktlen |= (hdr[hdrlen++] = iobuf_get_noeof(inp)) << 8;
 	    if( (c = iobuf_get(inp)) == -1 ) {
 		log_error("%s: 4 byte length invalid\n", iobuf_where(inp) );
-		return G10ERR_INVALID_PACKET;
+		rc = G10ERR_INVALID_PACKET;
+		goto leave;
 	    }
 	    pktlen |= (hdr[hdrlen++] = c );
 	}
 	else { /* partial body length */
-	    log_debug("partial body length of %lu bytes\n", pktlen );
-	    iobuf_set_partial_block_mode(inp, pktlen);
+	    iobuf_set_partial_block_mode(inp, c & 0xff);
 	    pktlen = 0;/* to indicate partial length */
 	}
     }
@@ -262,30 +283,31 @@ parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
 	    rc = G10ERR_WRITE_FILE;
 	else
 	    rc = copy_packet(inp, out, pkttype, pktlen );
-	return rc;
+	goto leave;
     }
 
     if( do_skip || !pkttype || (reqtype && pkttype != reqtype) ) {
 	skip_packet(inp, pkttype, pktlen);
 	*skip = 1;
-	return 0;
+	rc = 0;
+	goto leave;
     }
 
     if( DBG_PACKET )
 	log_debug("parse_packet(iob=%d): type=%d length=%lu%s\n",
-		   iobuf_id(inp), pkttype, pktlen, pgp3?" (pgp3)":"" );
+		   iobuf_id(inp), pkttype, pktlen, new_ctb?" (new_ctb)":"" );
     pkt->pkttype = pkttype;
     rc = G10ERR_UNKNOWN_PACKET; /* default error */
     switch( pkttype ) {
-      case PKT_PUBLIC_CERT:
-      case PKT_PUBKEY_SUBCERT:
-	pkt->pkt.public_cert = m_alloc_clear(sizeof *pkt->pkt.public_cert );
-	rc = parse_certificate(inp, pkttype, pktlen, hdr, hdrlen, pkt );
+      case PKT_PUBLIC_KEY:
+      case PKT_PUBLIC_SUBKEY:
+	pkt->pkt.public_key = m_alloc_clear(sizeof *pkt->pkt.public_key );
+	rc = parse_key(inp, pkttype, pktlen, hdr, hdrlen, pkt );
 	break;
-      case PKT_SECRET_CERT:
-      case PKT_SECKEY_SUBCERT:
-	pkt->pkt.secret_cert = m_alloc_clear(sizeof *pkt->pkt.secret_cert );
-	rc = parse_certificate(inp, pkttype, pktlen, hdr, hdrlen, pkt );
+      case PKT_SECRET_KEY:
+      case PKT_SECRET_SUBKEY:
+	pkt->pkt.secret_key = m_alloc_clear(sizeof *pkt->pkt.secret_key );
+	rc = parse_key(inp, pkttype, pktlen, hdr, hdrlen, pkt );
 	break;
       case PKT_SYMKEY_ENC:
 	rc = parse_symkeyenc( inp, pkttype, pktlen, pkt );
@@ -310,22 +332,25 @@ parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
 	break;
       case PKT_RING_TRUST:
 	parse_trust(inp, pkttype, pktlen);
-	rc = 0;
+	rc = G10ERR_UNKNOWN_PACKET;
 	break;
       case PKT_PLAINTEXT:
-	rc = parse_plaintext(inp, pkttype, pktlen, pkt );
+	rc = parse_plaintext(inp, pkttype, pktlen, pkt, new_ctb );
 	break;
       case PKT_COMPRESSED:
-	rc = parse_compressed(inp, pkttype, pktlen, pkt );
+	rc = parse_compressed(inp, pkttype, pktlen, pkt, new_ctb );
 	break;
       case PKT_ENCRYPTED:
-	rc = parse_encrypted(inp, pkttype, pktlen, pkt );
+	rc = parse_encrypted(inp, pkttype, pktlen, pkt, new_ctb );
 	break;
       default:
 	skip_packet(inp, pkttype, pktlen);
 	break;
     }
 
+  leave:
+    if( rc == -1 && iobuf_error(inp) )
+	rc = G10ERR_INV_KEYRING;
     return rc;
 }
 
@@ -413,6 +438,28 @@ skip_rest( IOBUF inp, unsigned long pktlen )
     }
 }
 
+static void *
+read_rest( IOBUF inp, ulong *r_pktlen )
+{
+    byte *p;
+    int i;
+    size_t pktlen = *r_pktlen;
+
+    if( iobuf_in_block_mode(inp) ) {
+	log_error("read_rest: can't store stream data\n");
+	p = NULL;
+    }
+    else {
+	p = m_alloc( pktlen + 2 );
+	p[0] = pktlen >> 8;
+	p[1] = pktlen & 0xff;
+	for(i=2; pktlen; pktlen--, i++ )
+	    p[i] = iobuf_get(inp);
+    }
+    *r_pktlen = 0;
+    return p;
+}
+
 
 static int
 parse_symkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
@@ -516,16 +563,21 @@ parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
 	  k->version, k->pubkey_algo, (ulong)k->keyid[0], (ulong)k->keyid[1]);
 
     ndata = pubkey_get_nenc(k->pubkey_algo);
-    if( !ndata && list_mode )
-	printf("\tunsupported algorithm %d\n", k->pubkey_algo );
-
-    for( i=0; i < ndata; i++ ) {
-	n = pktlen;
-	k->data[i] = mpi_read(inp, &n, 0); pktlen -=n;
-	if( list_mode ) {
-	    printf("\tdata: ");
-	    mpi_print(stdout, k->data[i], mpi_print_mode );
-	    putchar('\n');
+    if( !ndata ) {
+	if( list_mode )
+	    printf("\tunsupported algorithm %d\n", k->pubkey_algo );
+	unknown_pubkey_warning( k->pubkey_algo );
+	k->data[0] = NULL;  /* no need to store the encrypted data */
+    }
+    else {
+	for( i=0; i < ndata; i++ ) {
+	    n = pktlen;
+	    k->data[i] = mpi_read(inp, &n, 0); pktlen -=n;
+	    if( list_mode ) {
+		printf("\tdata: ");
+		mpi_print(stdout, k->data[i], mpi_print_mode );
+		putchar('\n');
+	    }
 	}
     }
 
@@ -751,17 +803,24 @@ parse_signature( IOBUF inp, int pkttype, unsigned long pktlen,
     }
 
     ndata = pubkey_get_nsig(sig->pubkey_algo);
-    if( !ndata && list_mode )
-	printf("\tunknown algorithm %d\n", sig->pubkey_algo );
-
-    for( i=0; i < ndata; i++ ) {
-	n = pktlen;
-	sig->data[i] = mpi_read(inp, &n, 0 );
-	pktlen -=n;
-	if( list_mode ) {
-	    printf("\tdata: ");
-	    mpi_print(stdout, sig->data[i], mpi_print_mode );
-	    putchar('\n');
+    if( !ndata ) {
+	if( list_mode )
+	    printf("\tunknown algorithm %d\n", sig->pubkey_algo );
+	unknown_pubkey_warning( sig->pubkey_algo );
+	/* we store the plain material in data[0], so that we are able
+	 * to write it back with build_packet() */
+	sig->data[0] = read_rest(inp, &pktlen );
+    }
+    else {
+	for( i=0; i < ndata; i++ ) {
+	    n = pktlen;
+	    sig->data[i] = mpi_read(inp, &n, 0 );
+	    pktlen -=n;
+	    if( list_mode ) {
+		printf("\tdata: ");
+		mpi_print(stdout, sig->data[i], mpi_print_mode );
+		putchar('\n');
+	    }
 	}
     }
 
@@ -810,18 +869,19 @@ parse_onepass_sig( IOBUF inp, int pkttype, unsigned long pktlen,
 
 
 static int
-parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
+parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 			      byte *hdr, int hdrlen, PACKET *pkt )
 {
     int i, version, algorithm;
     unsigned n;
     unsigned long timestamp;
     unsigned short valid_period;
+    int npkey, nskey;
     int is_v4=0;
     int rc=0;
 
     version = iobuf_get_noeof(inp); pktlen--;
-    if( pkttype == PKT_PUBKEY_SUBCERT && version == '#' ) {
+    if( pkttype == PKT_PUBLIC_SUBKEY && version == '#' ) {
 	/* early versions of G10 use old PGP comments packets;
 	 * luckily all those comments are started by a hash */
 	if( list_mode ) {
@@ -861,343 +921,185 @@ parse_certificate( IOBUF inp, int pkttype, unsigned long pktlen,
     if( list_mode )
 	printf(":%s key packet:\n"
 	       "\tversion %d, algo %d, created %lu, valid for %hu days\n",
-		pkttype == PKT_PUBLIC_CERT? "public" :
-		pkttype == PKT_SECRET_CERT? "secret" :
-		pkttype == PKT_PUBKEY_SUBCERT? "public sub" :
-		pkttype == PKT_SECKEY_SUBCERT? "secret sub" : "??",
+		pkttype == PKT_PUBLIC_KEY? "public" :
+		pkttype == PKT_SECRET_KEY? "secret" :
+		pkttype == PKT_PUBLIC_SUBKEY? "public sub" :
+		pkttype == PKT_SECRET_SUBKEY? "secret sub" : "??",
 		version, algorithm, timestamp, valid_period );
-    if( pkttype == PKT_SECRET_CERT || pkttype == PKT_SECKEY_SUBCERT )  {
-	pkt->pkt.secret_cert->timestamp = timestamp;
-	pkt->pkt.secret_cert->valid_days = valid_period;
-	pkt->pkt.secret_cert->hdrbytes = hdrlen;
-	pkt->pkt.secret_cert->version = version;
-	pkt->pkt.secret_cert->pubkey_algo = algorithm;
+
+    if( pkttype == PKT_SECRET_KEY || pkttype == PKT_SECRET_SUBKEY )  {
+	PKT_secret_key *sk = pkt->pkt.secret_key;
+
+	sk->timestamp = timestamp;
+	sk->valid_days = valid_period;
+	sk->hdrbytes = hdrlen;
+	sk->version = version;
+	sk->pubkey_algo = algorithm;
+	sk->pubkey_usage = 0; /* not yet used */
     }
     else {
-	pkt->pkt.public_cert->timestamp = timestamp;
-	pkt->pkt.public_cert->valid_days = valid_period;
-	pkt->pkt.public_cert->hdrbytes	  = hdrlen;
-	pkt->pkt.public_cert->version	  = version;
-	pkt->pkt.public_cert->pubkey_algo = algorithm;
+	PKT_public_key *pk = pkt->pkt.public_key;
+
+	pk->timestamp = timestamp;
+	pk->valid_days = valid_period;
+	pk->hdrbytes	= hdrlen;
+	pk->version	= version;
+	pk->pubkey_algo = algorithm;
+	pk->pubkey_usage = 0; /* not yet used */
+    }
+    nskey = pubkey_get_nskey( algorithm );
+    npkey = pubkey_get_npkey( algorithm );
+    if( !npkey ) {
+	if( list_mode )
+	    printf("\tunknown algorithm %d\n", algorithm );
+	unknown_pubkey_warning( algorithm );
     }
 
-    if( is_ELGAMAL(algorithm) ) {
-	MPI elg_p, elg_g, elg_y;
-	n = pktlen; elg_p = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen; elg_g = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen; elg_y = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf(  "\telg p: ");
-	    mpi_print(stdout, elg_p, mpi_print_mode  );
-	    printf("\n\telg g: ");
-	    mpi_print(stdout, elg_g, mpi_print_mode  );
-	    printf("\n\telg y: ");
-	    mpi_print(stdout, elg_y, mpi_print_mode  );
-	    putchar('\n');
+
+    if( pkttype == PKT_SECRET_KEY || pkttype == PKT_SECRET_SUBKEY )  {
+	PKT_secret_key *sk = pkt->pkt.secret_key;
+	byte temp[8];
+
+	if( !npkey ) {
+	    sk->skey[0] = read_rest( inp, &pktlen );
+	    goto leave;
 	}
-	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->pkey[0] = elg_p;
-	    pkt->pkt.public_cert->pkey[1] = elg_g;
-	    pkt->pkt.public_cert->pkey[2] = elg_y;
+
+	for(i=0; i < npkey; i++ ) {
+	    n = pktlen; sk->skey[i] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    if( list_mode ) {
+		printf(  "\tskey[%d]: ", i);
+		mpi_print(stdout, sk->skey[i], mpi_print_mode  );
+		putchar('\n');
+	    }
 	}
-	else {
-	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
-	    byte temp[8];
-
-	    pkt->pkt.secret_cert->skey[0] = elg_p;
-	    pkt->pkt.secret_cert->skey[1] = elg_g;
-	    pkt->pkt.secret_cert->skey[2] = elg_y;
-	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
-	    if( cert->protect.algo ) {
-		cert->is_protected = 1;
-		cert->protect.s2k.count = 0;
-		if( cert->protect.algo == 255 ) {
-		    if( pktlen < 3 ) {
-			rc = G10ERR_INVALID_PACKET;
-			goto leave;
-		    }
-		    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
-		    cert->protect.s2k.mode  = iobuf_get_noeof(inp); pktlen--;
-		    cert->protect.s2k.hash_algo = iobuf_get_noeof(inp); pktlen--;
-		    switch( cert->protect.s2k.mode ) {
-		      case 1:
-		      case 4:
-			for(i=0; i < 8 && pktlen; i++, pktlen-- )
-			    temp[i] = iobuf_get_noeof(inp);
-			memcpy(cert->protect.s2k.salt, temp, 8 );
-			break;
-		    }
-		    switch( cert->protect.s2k.mode ) {
-		      case 0: if( list_mode ) printf(  "\tsimple S2K" );
-			break;
-		      case 1: if( list_mode ) printf(  "\tsalted S2K" );
-			break;
-		      case 4: if( list_mode ) printf(  "\titer+salt S2K" );
-			break;
-		      default:
-			if( list_mode )
-			    printf(  "\tunknown S2K %d\n",
-						cert->protect.s2k.mode );
-			rc = G10ERR_INVALID_PACKET;
-			goto leave;
-		    }
-
-		    if( list_mode ) {
-			printf(", algo: %d, hash: %d",
-					 cert->protect.algo,
-					 cert->protect.s2k.hash_algo );
-			if( cert->protect.s2k.mode == 1
-			    || cert->protect.s2k.mode == 4 ) {
-			    printf(", salt: ");
-			    for(i=0; i < 8; i++ )
-				printf("%02x", cert->protect.s2k.salt[i]);
-			}
-			putchar('\n');
-		    }
-
-		    if( cert->protect.s2k.mode == 4 ) {
-			if( pktlen < 4 ) {
-			    rc = G10ERR_INVALID_PACKET;
-			    goto leave;
-			}
-			cert->protect.s2k.count = read_32(inp);
-			pktlen -= 4;
-		    }
-
-		}
-		else {
-		    /* old version, we don't have a S2K, so we fake one */
-		    cert->protect.s2k.mode = 0;
-		    /* We need this kludge to cope with old GNUPG versions */
-		    cert->protect.s2k.hash_algo =
-			 cert->protect.algo == CIPHER_ALGO_BLOWFISH160?
-				      DIGEST_ALGO_RMD160 : DIGEST_ALGO_MD5;
-		    if( list_mode )
-			printf(  "\tprotect algo: %d  (hash algo: %d)\n",
-			     cert->protect.algo, cert->protect.s2k.hash_algo );
-		}
-		if( pktlen < 8 ) {
+	sk->protect.algo = iobuf_get_noeof(inp); pktlen--;
+	if( sk->protect.algo ) {
+	    sk->is_protected = 1;
+	    sk->protect.s2k.count = 0;
+	    if( sk->protect.algo == 255 ) {
+		if( pktlen < 3 ) {
 		    rc = G10ERR_INVALID_PACKET;
 		    goto leave;
 		}
-		for(i=0; i < 8 && pktlen; i++, pktlen-- )
-		    temp[i] = iobuf_get_noeof(inp);
-		if( list_mode ) {
-		    printf(  "\tprotect IV: ");
-		    for(i=0; i < 8; i++ )
-			printf(" %02x", temp[i] );
-		    putchar('\n');
+		sk->protect.algo = iobuf_get_noeof(inp); pktlen--;
+		sk->protect.s2k.mode  = iobuf_get_noeof(inp); pktlen--;
+		sk->protect.s2k.hash_algo = iobuf_get_noeof(inp); pktlen--;
+		switch( sk->protect.s2k.mode ) {
+		  case 1:
+		  case 4:
+		    for(i=0; i < 8 && pktlen; i++, pktlen-- )
+			temp[i] = iobuf_get_noeof(inp);
+		    memcpy(sk->protect.s2k.salt, temp, 8 );
+		    break;
 		}
-		memcpy(cert->protect.iv, temp, 8 );
-	    }
-	    else
-		cert->is_protected = 0;
-	    /* It does not make sense to read it into secure memory.
-	     * If the user is so careless, not to protect his secret key,
-	     * we can assume, that he operates an open system :=(.
-	     * So we put the key into secure memory when we unprotect it. */
-	    n = pktlen; cert->skey[3] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    if( cert->is_protected )
-		mpi_set_protect_flag(cert->skey[3]);
-
-	    cert->csum = read_16(inp); pktlen -= 2;
-	    if( list_mode ) {
-		printf("\t[secret value x is not shown]\n"
-		       "\tchecksum: %04hx\n", cert->csum);
-	    }
-	}
-    }
-    else if( algorithm == PUBKEY_ALGO_DSA ) {
-	MPI dsa_p, dsa_q, dsa_g, dsa_y;
-	n = pktlen; dsa_p = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen; dsa_q = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen; dsa_g = mpi_read(inp, &n, 0 ); pktlen -=n;
-	n = pktlen; dsa_y = mpi_read(inp, &n, 0 ); pktlen -=n;
-	if( list_mode ) {
-	    printf(  "\tdsa p: ");
-	    mpi_print(stdout, dsa_p, mpi_print_mode  );
-	    printf("\n\tdsa q: ");
-	    mpi_print(stdout, dsa_q, mpi_print_mode  );
-	    printf("\n\tdsa g: ");
-	    mpi_print(stdout, dsa_g, mpi_print_mode  );
-	    printf("\n\tdsa y: ");
-	    mpi_print(stdout, dsa_y, mpi_print_mode  );
-	    putchar('\n');
-	}
-	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->pkey[0] = dsa_p;
-	    pkt->pkt.public_cert->pkey[1] = dsa_q;
-	    pkt->pkt.public_cert->pkey[2] = dsa_g;
-	    pkt->pkt.public_cert->pkey[3] = dsa_y;
-	}
-	else {
-	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
-	    byte temp[8];
-
-	    pkt->pkt.secret_cert->skey[0] = dsa_p;
-	    pkt->pkt.secret_cert->skey[1] = dsa_q;
-	    pkt->pkt.secret_cert->skey[2] = dsa_g;
-	    pkt->pkt.secret_cert->skey[3] = dsa_y;
-	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
-	    if( cert->protect.algo ) {
-		cert->is_protected = 1;
-		cert->protect.s2k.count = 0;
-		if( cert->protect.algo == 255 ) {
-		    if( pktlen < 3 ) {
-			rc = G10ERR_INVALID_PACKET;
-			goto leave;
-		    }
-		    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
-		    cert->protect.s2k.mode  = iobuf_get_noeof(inp); pktlen--;
-		    cert->protect.s2k.hash_algo = iobuf_get_noeof(inp); pktlen--;
-		    switch( cert->protect.s2k.mode ) {
-		      case 1:
-		      case 4:
-			for(i=0; i < 8 && pktlen; i++, pktlen-- )
-			    temp[i] = iobuf_get_noeof(inp);
-			memcpy(cert->protect.s2k.salt, temp, 8 );
-			break;
-		    }
-		    switch( cert->protect.s2k.mode ) {
-		      case 0: if( list_mode ) printf(  "\tsimple S2K" );
-			break;
-		      case 1: if( list_mode ) printf(  "\tsalted S2K" );
-			break;
-		      case 4: if( list_mode ) printf(  "\titer+salt S2K" );
-			break;
-		      default:
-			if( list_mode )
-			    printf(  "\tunknown S2K %d\n",
-						    cert->protect.s2k.mode );
-			rc = G10ERR_INVALID_PACKET;
-			goto leave;
-		    }
-
-		    if( list_mode ) {
-			printf(", algo: %d, hash: %d",
-					 cert->protect.algo,
-					 cert->protect.s2k.hash_algo );
-			if( cert->protect.s2k.mode == 1
-			    || cert->protect.s2k.mode == 4 ){
-			    printf(", salt: ");
-			    for(i=0; i < 8; i++ )
-				printf("%02x", cert->protect.s2k.salt[i]);
-			}
-			putchar('\n');
-		    }
-
-		    if( cert->protect.s2k.mode == 4 ) {
-			if( pktlen < 4 ) {
-			    rc = G10ERR_INVALID_PACKET;
-			    goto leave;
-			}
-			cert->protect.s2k.count = read_32(inp);
-			pktlen -= 4;
-		    }
-
-		}
-		else {
+		switch( sk->protect.s2k.mode ) {
+		  case 0: if( list_mode ) printf(  "\tsimple S2K" );
+		    break;
+		  case 1: if( list_mode ) printf(  "\tsalted S2K" );
+		    break;
+		  case 4: if( list_mode ) printf(  "\titer+salt S2K" );
+		    break;
+		  default:
 		    if( list_mode )
-			printf(  "\tprotect algo: %d\n", cert->protect.algo);
-		    /* old version, we don't have a S2K, so we fake one */
-		    cert->protect.s2k.mode = 0;
-		    cert->protect.s2k.hash_algo = DIGEST_ALGO_MD5;
-		}
-		if( pktlen < 8 ) {
+			printf(  "\tunknown S2K %d\n",
+					    sk->protect.s2k.mode );
 		    rc = G10ERR_INVALID_PACKET;
 		    goto leave;
 		}
-		for(i=0; i < 8 && pktlen; i++, pktlen-- )
-		    temp[i] = iobuf_get_noeof(inp);
+
 		if( list_mode ) {
-		    printf(  "\tprotect IV: ");
-		    for(i=0; i < 8; i++ )
-			printf(" %02x", temp[i] );
+		    printf(", algo: %d, hash: %d",
+				     sk->protect.algo,
+				     sk->protect.s2k.hash_algo );
+		    if( sk->protect.s2k.mode == 1
+			|| sk->protect.s2k.mode == 4 ) {
+			printf(", salt: ");
+			for(i=0; i < 8; i++ )
+			    printf("%02x", sk->protect.s2k.salt[i]);
+		    }
 		    putchar('\n');
 		}
-		memcpy(cert->protect.iv, temp, 8 );
-	    }
-	    else
-		cert->is_protected = 0;
-	    /* It does not make sense to read it into secure memory.
-	     * If the user is so careless, not to protect his secret key,
-	     * we can assume, that he operates an open system :=(.
-	     * So we put the key into secure memory when we unprotect it. */
-	    n = pktlen; cert->skey[4] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    if( cert->is_protected )
-		mpi_set_protect_flag(cert->skey[4]);
 
-	    cert->csum = read_16(inp); pktlen -= 2;
+		if( sk->protect.s2k.mode == 4 ) {
+		    if( pktlen < 4 ) {
+			rc = G10ERR_INVALID_PACKET;
+			goto leave;
+		    }
+		    sk->protect.s2k.count = read_32(inp);
+		    pktlen -= 4;
+		}
+
+	    }
+	    else { /* old version; no S2K, so we set mode to 0, hash MD5 */
+		sk->protect.s2k.mode = 0;
+		/* We need a kludge to cope with old GNUPG versions */
+		sk->protect.s2k.hash_algo =
+			 ( sk->protect.algo == CIPHER_ALGO_BLOWFISH160
+			   && algorithm == PUBKEY_ALGO_ELGAMAL_E ) ?
+				  DIGEST_ALGO_RMD160 : DIGEST_ALGO_MD5;
+		if( list_mode )
+		    printf(  "\tprotect algo: %d  (hash algo: %d)\n",
+			 sk->protect.algo, sk->protect.s2k.hash_algo );
+	    }
+	    if( pktlen < 8 ) {
+		rc = G10ERR_INVALID_PACKET;
+		goto leave;
+	    }
+	    for(i=0; i < 8 && pktlen; i++, pktlen-- )
+		temp[i] = iobuf_get_noeof(inp);
 	    if( list_mode ) {
-		printf("\t[secret value x is not shown]\n"
-		       "\tchecksum: %04hx\n", cert->csum);
+		printf(  "\tprotect IV: ");
+		for(i=0; i < 8; i++ )
+		    printf(" %02x", temp[i] );
+		putchar('\n');
+	    }
+	    memcpy(sk->protect.iv, temp, 8 );
+	}
+	else
+	    sk->is_protected = 0;
+	/* It does not make sense to read it into secure memory.
+	 * If the user is so careless, not to protect his secret key,
+	 * we can assume, that he operates an open system :=(.
+	 * So we put the key into secure memory when we unprotect it. */
+
+	for(i=npkey; i < nskey; i++ ) {
+	    n = pktlen; sk->skey[i] = mpi_read(inp, &n, 0 ); pktlen -=n;
+	    if( sk->is_protected )
+		mpi_set_protect_flag(sk->skey[i]);
+	    if( list_mode ) {
+		printf(  "\tskey[%d]: ", i);
+		if( sk->is_protected )
+		    printf(  "[encrypted]\n");
+		else {
+		    mpi_print(stdout, sk->skey[i], mpi_print_mode  );
+		    putchar('\n');
+		}
 	    }
 	}
-    }
-    else if( is_RSA(algorithm) ) {
-	MPI rsa_pub_mod, rsa_pub_exp;
 
-	n = pktlen; rsa_pub_mod = mpi_read(inp, &n, 0); pktlen -=n;
-	n = pktlen; rsa_pub_exp = mpi_read(inp, &n, 0 ); pktlen -=n;
+	sk->csum = read_16(inp); pktlen -= 2;
 	if( list_mode ) {
-	    printf(  "\tpublic modulus  n:  ");
-	    mpi_print(stdout, rsa_pub_mod, mpi_print_mode  );
-	    printf("\n\tpublic exponent e: ");
-	    mpi_print(stdout, rsa_pub_exp, mpi_print_mode  );
-	    putchar('\n');
+	    printf("\tchecksum: %04hx\n", sk->csum);
 	}
-	if( pkttype == PKT_PUBLIC_CERT || pkttype == PKT_PUBKEY_SUBCERT ) {
-	    pkt->pkt.public_cert->pkey[0] = rsa_pub_mod;
-	    pkt->pkt.public_cert->pkey[1] = rsa_pub_exp;
+    }
+    else {
+	PKT_public_key *pk = pkt->pkt.public_key;
+
+	if( !npkey ) {
+	    pk->pkey[0] = read_rest( inp, &pktlen );
+	    goto leave;
 	}
-	else {
-	    PKT_secret_cert *cert = pkt->pkt.secret_cert;
-	    byte temp[8];
 
-	    pkt->pkt.secret_cert->skey[0] = rsa_pub_mod;
-	    pkt->pkt.secret_cert->skey[1] = rsa_pub_exp;
-	    cert->protect.algo = iobuf_get_noeof(inp); pktlen--;
-	    if( list_mode )
-		printf(  "\tprotect algo: %d\n", cert->protect.algo);
-	    if( cert->protect.algo ) {
-		cert->is_protected = 1;
-		for(i=0; i < 8 && pktlen; i++, pktlen-- )
-		    temp[i] = iobuf_get_noeof(inp);
-		if( list_mode ) {
-		    printf(  "\tprotect IV: ");
-		    for(i=0; i < 8; i++ )
-			printf(" %02x", temp[i] );
-		    putchar('\n');
-		}
-		memcpy(cert->protect.iv, temp, 8 );
-		/* old version, we don't have a S2K, so we fake one */
-		cert->protect.s2k.mode = 0;
-		cert->protect.s2k.hash_algo = DIGEST_ALGO_MD5;
-	    }
-	    else
-		cert->is_protected = 0;
-	    /* (See comments at the code for elg keys) */
-	    n = pktlen; cert->skey[2] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->skey[3] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->skey[4] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    n = pktlen; cert->skey[5] = mpi_read(inp, &n, 0 ); pktlen -=n;
-	    if( cert->is_protected ) {
-		mpi_set_protect_flag(cert->skey[2]);
-		mpi_set_protect_flag(cert->skey[3]);
-		mpi_set_protect_flag(cert->skey[4]);
-		mpi_set_protect_flag(cert->skey[5]);
-	    }
-
-	    cert->csum = read_16(inp); pktlen -= 2;
+	for(i=0; i < npkey; i++ ) {
+	    n = pktlen; pk->pkey[i] = mpi_read(inp, &n, 0 ); pktlen -=n;
 	    if( list_mode ) {
-		printf("\t[secret values d,p,q,u are not shown]\n"
-		       "\tchecksum: %04hx\n", cert->csum);
+		printf(  "\tpkey[%d]: ", i);
+		mpi_print(stdout, pk->pkey[i], mpi_print_mode  );
+		putchar('\n');
 	    }
 	}
     }
-    else if( list_mode )
-	printf("\tunknown algorithm %d\n", algorithm );
-
 
   leave:
     skip_rest(inp, pktlen);
@@ -1271,7 +1173,8 @@ parse_trust( IOBUF inp, int pkttype, unsigned long pktlen )
 
 
 static int
-parse_plaintext( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
+parse_plaintext( IOBUF inp, int pkttype, unsigned long pktlen,
+					PACKET *pkt, int new_ctb )
 {
     int mode, namelen;
     PKT_plaintext *pt;
@@ -1285,6 +1188,7 @@ parse_plaintext( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
     mode = iobuf_get_noeof(inp); if( pktlen ) pktlen--;
     namelen = iobuf_get_noeof(inp); if( pktlen ) pktlen--;
     pt = pkt->pkt.plaintext = m_alloc(sizeof *pkt->pkt.plaintext + namelen -1);
+    pt->new_ctb = new_ctb;
     pt->mode = mode;
     pt->namelen = namelen;
     if( pktlen ) {
@@ -1323,7 +1227,8 @@ parse_plaintext( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
 
 
 static int
-parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
+parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen,
+		  PACKET *pkt, int new_ctb )
 {
     PKT_compressed *zd;
 
@@ -1334,6 +1239,7 @@ parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
     zd = pkt->pkt.compressed =	m_alloc(sizeof *pkt->pkt.compressed );
     zd->len = 0; /* not yet used */
     zd->algorithm = iobuf_get_noeof(inp);
+    zd->new_ctb = new_ctb;
     zd->buf = inp;
     if( list_mode )
 	printf(":compressed packet: algo=%d\n", zd->algorithm);
@@ -1342,13 +1248,15 @@ parse_compressed( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
 
 
 static int
-parse_encrypted( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *pkt )
+parse_encrypted( IOBUF inp, int pkttype, unsigned long pktlen,
+				       PACKET *pkt, int new_ctb )
 {
     PKT_encrypted *ed;
 
     ed = pkt->pkt.encrypted =  m_alloc(sizeof *pkt->pkt.encrypted );
     ed->len = pktlen;
     ed->buf = NULL;
+    ed->new_ctb = new_ctb;
     if( pktlen && pktlen < 10 ) {
 	log_error("packet(%d) too short\n", pkttype);
 	skip_rest(inp, pktlen);
