@@ -39,13 +39,16 @@
 
 
 static int
-do_sign( PKT_secret_cert *skc, PKT_signature *sig,
+do_sign( PKT_secret_key *sk, PKT_signature *sig,
 	 MD_HANDLE md, int digest_algo )
 {
     MPI frame;
     byte *dp;
     int rc;
 
+    if( is_RSA(sk->pubkey_algo) )
+	do_not_use_RSA();
+
     if( !digest_algo )
 	digest_algo = md_get_algo(md);
 
@@ -53,9 +56,9 @@ do_sign( PKT_secret_cert *skc, PKT_signature *sig,
     sig->digest_algo = digest_algo;
     sig->digest_start[0] = dp[0];
     sig->digest_start[1] = dp[1];
-    frame = encode_md_value( skc->pubkey_algo, md,
-			     digest_algo, mpi_get_nbits(skc->skey[0]));
-    rc = pubkey_sign( skc->pubkey_algo, sig->data, frame, skc->skey );
+    frame = encode_md_value( sk->pubkey_algo, md,
+			     digest_algo, mpi_get_nbits(sk->skey[0]));
+    rc = pubkey_sign( sk->pubkey_algo, sig->data, frame, sk->skey );
     mpi_free(frame);
     if( rc )
 	log_error("pubkey_sign failed: %s\n", g10_errstr(rc) );
@@ -63,7 +66,7 @@ do_sign( PKT_secret_cert *skc, PKT_signature *sig,
 	if( opt.verbose ) {
 	    char *ustr = get_user_id_string( sig->keyid );
 	    log_info("%s signature from: %s\n",
-		      pubkey_algo_to_string(skc->pubkey_algo), ustr );
+		      pubkey_algo_to_string(sk->pubkey_algo), ustr );
 	    m_free(ustr);
 	}
     }
@@ -73,12 +76,12 @@ do_sign( PKT_secret_cert *skc, PKT_signature *sig,
 
 
 int
-complete_sig( PKT_signature *sig, PKT_secret_cert *skc, MD_HANDLE md )
+complete_sig( PKT_signature *sig, PKT_secret_key *sk, MD_HANDLE md )
 {
     int rc=0;
 
-    if( !(rc=check_secret_key( skc )) )
-	rc = do_sign( skc, sig, md, 0 );
+    if( !(rc=check_secret_key( sk )) )
+	rc = do_sign( sk, sig, md, 0 );
 
     /* fixme: should we check whether the signature is okay?
      * maybe by using an option */
@@ -99,15 +102,15 @@ hash_for(int pubkey_algo )
 }
 
 static int
-only_old_style( SKC_LIST skc_list )
+only_old_style( SK_LIST sk_list )
 {
-    SKC_LIST skc_rover = NULL;
+    SK_LIST sk_rover = NULL;
     int old_style = 0;
 
     /* if there are only old style capable key we use the old sytle */
-    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	PKT_secret_cert *skc = skc_rover->skc;
-	if( skc->pubkey_algo == PUBKEY_ALGO_RSA && skc->version < 4 )
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk = sk_rover->sk;
+	if( sk->pubkey_algo == PUBKEY_ALGO_RSA && sk->version < 4 )
 	    old_style = 1;
 	else
 	    return 0;
@@ -142,9 +145,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     PKT_plaintext *pt = NULL;
     u32 filesize;
     int rc = 0;
-    PKC_LIST pkc_list = NULL;
-    SKC_LIST skc_list = NULL;
-    SKC_LIST skc_rover = NULL;
+    PK_LIST pk_list = NULL;
+    SK_LIST sk_list = NULL;
+    SK_LIST sk_rover = NULL;
     int multifile = 0;
     int old_style = opt.rfc1991;
 
@@ -166,12 +169,12 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     if( fname && filenames->next && (!detached || encrypt) )
 	log_bug("multiple files can only be detached signed");
 
-    if( (rc=build_skc_list( locusr, &skc_list, 1, 1 )) )
+    if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
 	goto leave;
     if( !old_style )
-	old_style = only_old_style( skc_list );
+	old_style = only_old_style( sk_list );
     if( encrypt ) {
-	if( (rc=build_pkc_list( remusr, &pkc_list, 2 )) )
+	if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC )) )
 	    goto leave;
     }
 
@@ -204,9 +207,9 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_push_filter( inp, text_filter, &tfx );
     mfx.md = md_open(0, 0);
 
-    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	PKT_secret_cert *skc = skc_rover->skc;
-	md_enable(mfx.md, hash_for(skc->pubkey_algo));
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk = sk_rover->sk;
+	md_enable(mfx.md, hash_for(sk->pubkey_algo));
     }
 
     if( !multifile )
@@ -218,7 +221,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	write_comment( out, "#created by GNUPG v" VERSION " ("
 					    PRINTABLE_OS_NAME ")");
     if( encrypt ) {
-	efx.pkc_list = pkc_list;
+	efx.pk_list = pk_list;
 	/* fixme: set efx.cfx.datalen if known */
 	iobuf_push_filter( out, encrypt_filter, &efx );
     }
@@ -231,17 +234,17 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 
     if( !detached && !old_style ) {
 	/* loop over the secret certificates and build headers */
-	for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	    PKT_secret_cert *skc;
+	for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	    PKT_secret_key *sk;
 	    PKT_onepass_sig *ops;
 
-	    skc = skc_rover->skc;
+	    sk = sk_rover->sk;
 	    ops = m_alloc_clear( sizeof *ops );
 	    ops->sig_class = opt.textmode && !outfile ? 0x01 : 0x00;
-	    ops->digest_algo = hash_for(skc->pubkey_algo);
-	    ops->pubkey_algo = skc->pubkey_algo;
-	    keyid_from_skc( skc, ops->keyid );
-	    ops->last = !skc_rover->next;
+	    ops->digest_algo = hash_for(sk->pubkey_algo);
+	    ops->pubkey_algo = sk->pubkey_algo;
+	    keyid_from_sk( sk, ops->keyid );
+	    ops->last = !sk_rover->next;
 
 	    init_packet(&pkt);
 	    pkt.pkttype = PKT_ONEPASS_SIG;
@@ -314,6 +317,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	pt->timestamp = make_timestamp();
 	pt->mode = opt.textmode && !outfile ? 't':'b';
 	pt->len = filesize;
+	pt->new_ctb = !pt->len && !opt.rfc1991;
 	pt->buf = inp;
 	pkt.pkttype = PKT_PLAINTEXT;
 	pkt.pkt.plaintext = pt;
@@ -324,20 +328,20 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
     }
 
     /* loop over the secret certificates */
-    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	PKT_secret_cert *skc;
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk;
 	PKT_signature *sig;
 	MD_HANDLE md;
 
-	skc = skc_rover->skc;
+	sk = sk_rover->sk;
 
 	/* build the signature packet */
 	/* fixme: this code is partly duplicated in make_keysig_packet */
 	sig = m_alloc_clear( sizeof *sig );
-	sig->version = skc->version;
-	keyid_from_skc( skc, sig->keyid );
-	sig->digest_algo = hash_for(skc->pubkey_algo);
-	sig->pubkey_algo = skc->pubkey_algo;
+	sig->version = sk->version;
+	keyid_from_sk( sk, sig->keyid );
+	sig->digest_algo = hash_for(sk->pubkey_algo);
+	sig->pubkey_algo = sk->pubkey_algo;
 	sig->timestamp = make_timestamp();
 	sig->sig_class = opt.textmode && !outfile? 0x01 : 0x00;
 
@@ -380,7 +384,7 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	}
 	md_final( md );
 
-	rc = do_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
+	rc = do_sign( sk, sig, md, hash_for(sig->pubkey_algo) );
 	md_close( md );
 
 	if( !rc ) { /* and write it */
@@ -404,8 +408,8 @@ sign_file( STRLIST filenames, int detached, STRLIST locusr,
 	iobuf_close(out);
     iobuf_close(inp);
     md_close( mfx.md );
-    release_skc_list( skc_list );
-    release_pkc_list( pkc_list );
+    release_sk_list( sk_list );
+    release_pk_list( pk_list );
     return rc;
 }
 
@@ -458,18 +462,19 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     IOBUF inp = NULL, out = NULL;
     PACKET pkt;
     int rc = 0;
-    SKC_LIST skc_list = NULL;
-    SKC_LIST skc_rover = NULL;
+    SK_LIST sk_list = NULL;
+    SK_LIST sk_rover = NULL;
     int old_style = opt.rfc1991;
+    int only_md5 = 0;
 
     memset( &afx, 0, sizeof afx);
     memset( &tfx, 0, sizeof tfx);
     init_packet( &pkt );
 
-    if( (rc=build_skc_list( locusr, &skc_list, 1, 1 )) )
+    if( (rc=build_sk_list( locusr, &sk_list, 1, PUBKEY_USAGE_SIG )) )
 	goto leave;
     if( !old_style )
-	old_style = only_old_style( skc_list );
+	old_style = only_old_style( sk_list );
 
     /* prepare iobufs */
     if( !(inp = iobuf_open(fname)) ) {
@@ -493,26 +498,44 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	goto leave;
     }
 
-    /* FIXME: This stuff is not correct if multiple hash algos are used*/
     iobuf_writestr(out, "-----BEGIN PGP SIGNED MESSAGE-----\n" );
-    if( old_style
-	|| (opt.def_digest_algo?opt.def_digest_algo:DEFAULT_DIGEST_ALGO)
-			      == DIGEST_ALGO_MD5 )
+
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk = sk_rover->sk;
+	if( hash_for(sk->pubkey_algo) == DIGEST_ALGO_MD5 )
+	    only_md5 = 1;
+	else {
+	    only_md5 = 0;
+	    break;
+	}
+    }
+
+    if( old_style || only_md5 )
 	iobuf_writestr(out, "\n" );
     else {
-	const char *s = digest_algo_to_string(opt.def_digest_algo?
-				    opt.def_digest_algo:DEFAULT_DIGEST_ALGO);
-	assert(s);
+	const char *s;
+	int any = 0;
+
 	iobuf_writestr(out, "Hash: " );
-	iobuf_writestr(out, s );
+	for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	    PKT_secret_key *sk = sk_rover->sk;
+	    s = digest_algo_to_string( hash_for(sk->pubkey_algo) );
+	    if( s ) {
+		if( any )
+		    iobuf_put(out, ',' );
+		iobuf_writestr(out, s );
+		any = 1;
+	    }
+	}
+	assert(any);
 	iobuf_writestr(out, "\n\n" );
     }
 
 
     textmd = md_open(0, 0);
-    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	PKT_secret_cert *skc = skc_rover->skc;
-	md_enable(textmd, hash_for(skc->pubkey_algo));
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk = sk_rover->sk;
+	md_enable(textmd, hash_for(sk->pubkey_algo));
     }
 
     iobuf_push_filter( inp, text_filter, &tfx );
@@ -525,20 +548,20 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
     iobuf_push_filter( out, armor_filter, &afx );
 
     /* loop over the secret certificates */
-    for( skc_rover = skc_list; skc_rover; skc_rover = skc_rover->next ) {
-	PKT_secret_cert *skc;
+    for( sk_rover = sk_list; sk_rover; sk_rover = sk_rover->next ) {
+	PKT_secret_key *sk;
 	PKT_signature *sig;
 	MD_HANDLE md;
 
-	skc = skc_rover->skc;
+	sk = sk_rover->sk;
 
 	/* build the signature packet */
 	/* fixme: this code is duplicated above */
 	sig = m_alloc_clear( sizeof *sig );
-	sig->version = skc->version;
-	keyid_from_skc( skc, sig->keyid );
-	sig->digest_algo = hash_for(skc->pubkey_algo);
-	sig->pubkey_algo = skc->pubkey_algo;
+	sig->version = sk->version;
+	keyid_from_sk( sk, sig->keyid );
+	sig->digest_algo = hash_for(sk->pubkey_algo);
+	sig->pubkey_algo = sk->pubkey_algo;
 	sig->timestamp = make_timestamp();
 	sig->sig_class = 0x01;
 
@@ -580,7 +603,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	}
 	md_final( md );
 
-	rc = do_sign( skc, sig, md, hash_for(sig->pubkey_algo) );
+	rc = do_sign( sk, sig, md, hash_for(sig->pubkey_algo) );
 	md_close( md );
 
 	if( !rc ) { /* and write it */
@@ -604,7 +627,7 @@ clearsign_file( const char *fname, STRLIST locusr, const char *outfile )
 	iobuf_close(out);
     iobuf_close(inp);
     md_close( textmd );
-    release_skc_list( skc_list );
+    release_sk_list( sk_list );
     return rc;
 }
 
